% Calculating business cycle facts for US Macroeconomic time series
% Alejandro Riao
% Date: 21/08/2019

% clearing variables and cleaning screen
clc;
clear;

% loading data downloaded from St Louis Fed FRED
load BusinessCycleData_US

lambda_q = 1600;

% 1) plotting raw data (in logs)
figure(1)
GDPagg = [GDPlog Clog Investmentlog Glog Exportslog Importslog];
plot(Date,GDPagg)
title('Macroeconomic Aggregates (in logs)')
xlabel('Date')
legend('GDP','C','I','G','X','M','location','southeast')

% 2) calculating cyclical component of GDP components
GDPagg_t =  hpfilter(GDPagg,lambda_q);
GDPagg_c = GDPagg - GDPagg_t;

% 3) Plotting cyclical component of time-series against the cyclical
% component of GDP

figure(2)
title('Cyclical Component of Macroeconomic Aggregates')
subplot(2,3,1)
plot(Date,GDPagg_c(:,1),Date,GDPagg_c(:,2))
ylim([-0.3,0.3])
legend('GDP','C')
subplot(2,3,2)
plot(Date,GDPagg_c(:,1),Date,GDPagg_c(:,3))
ylim([-0.3,0.3])
legend('GDP','I')
subplot(2,3,3)
plot(Date,GDPagg_c(:,1),Date,GDPagg_c(:,3))
ylim([-0.3,0.3])
legend('GDP','G')
subplot(2,3,4)
plot(Date,GDPagg_c(:,1),Date,GDPagg_c(:,4))
ylim([-0.3,0.3])
legend('GDP','X')
subplot(2,3,5)
plot(Date,GDPagg_c(:,1),Date,GDPagg_c(:,5))
ylim([-0.3,0.3])
legend('GDP','M')

% 4) Relative standard deviation of cyclical components of GDP

SD = std(GDPagg_c);
RELSD = (SD./SD(1,1))';

% 5) Autocorrelation (persistence of cyclical components)
horizon = 3; % lead/lag horizon = 3 quarters
GDPagg_c_LF = intmat(GDPagg_c,horizon);

AUTOCORR = zeros(6,1);
for i=1:6
   int = corrcoef(GDPagg_c_LF(:,3,i),GDPagg_c_LF(:,4,i));
   AUTOCORR(i) = int(1,2);
end

% 5) Cross-correlation with GDP
CROSSCORR = zeros(6,7);
for i=1:6
    for t=1:(2*horizon +1)        
        int2 = corrcoef(GDPagg_c_LF(:,t,i),GDPagg_c_LF(:,4,1));
        CROSSCORR(i,t) = int2(1,2);
    end
end


% Displaying results:

disp('Results')

disp('-------------------------------------------------')
disp('Variable      Rel. Std. Dev        Autocorr' )
disp('-------------------------------------------------')
fprintf('Y \t\t\t\t %4.3f \t\t\t\t %8.3f \n',RELSD(1),AUTOCORR(1))
fprintf('C \t\t\t\t %4.3f \t\t\t\t %8.3f \n',RELSD(2),AUTOCORR(2))
fprintf('I \t\t\t\t %4.3f \t\t\t\t %8.3f \n',RELSD(3),AUTOCORR(3))
fprintf('G \t\t\t\t %4.3f \t\t\t\t %8.3f \n',RELSD(4),AUTOCORR(4))
fprintf('X \t\t\t\t %4.3f \t\t\t\t %8.3f \n',RELSD(5),AUTOCORR(5))
fprintf('M \t\t\t\t %4.3f \t\t\t\t %8.3f \n',RELSD(6),AUTOCORR(6))
disp('-------------------------------------------------')

heading = linspace(-horizon,horizon,2*horizon+1);
disp('--------------------------------------------------------------------------')
disp('Variable      Cross corr with cyclical component of GDP' )
disp('--------------------------------------------------------------------------')
fprintf('\t\t\t\t %4.0f \t %4.0f \t %4.0f \t %4.0f \t %4.0f \t %4.0f \t %4.0f \n',heading)
disp('--------------------------------------------------------------------------')
fprintf('Y \t\t\t\t %4.2f \t %4.2f \t %4.2f \t %4.2f \t %4.2f \t %4.2f \t %4.2f \n',CROSSCORR(1,:))
fprintf('C \t\t\t\t %4.2f \t %4.2f \t %4.2f \t %4.2f \t %4.2f \t %4.2f \t %4.2f \n',CROSSCORR(2,:))
fprintf('I \t\t\t\t %4.2f \t %4.2f \t %4.2f \t %4.2f \t %4.2f \t %4.2f \t %4.2f \n',CROSSCORR(3,:))
fprintf('G \t\t\t\t %4.2f \t %4.2f \t %4.2f \t %4.2f \t %4.2f \t %4.2f \t %4.2f \n',CROSSCORR(4,:))
fprintf('X \t\t\t\t %4.2f \t %4.2f \t %4.2f \t %4.2f \t %4.2f \t %4.2f \t %4.2f \n',CROSSCORR(5,:))
fprintf('M \t\t\t\t %4.2f \t %4.2f \t %4.2f \t %4.2f \t %4.2f \t %4.2f \t %4.2f \n',CROSSCORR(6,:))
disp('---------------------------------------------------------------------------')


